int itemNum = 250;
float friction = -0.1;
Item[] items = new Item[itemNum];
PVector v1e;
PVector v2e; 
void setup4() {
  size(640,520,OPENGL);
  smooth();
  frameRate(60);
  v1e = tracker.getPos();
  v2e = tracker.getLerpedPos();
  for (int i=0; i<itemNum; i++) {
    items[i] = new Item(0, 0, random(2, 4), i, items);
  }
  filter(BLUR,10);
}
 
void draw4() {
  tracker.track();
  v1e = tracker.getPos();
  v2e = tracker.getLerpedPos();
  background(20, 20, 30);
  fill(200, 100, 50);
  ellipseMode(CENTER);
  ellipse(v2e.x, v2e.y, 10, 10);
 
  for (int i = 0; i<itemNum; i++) {
    items[i].move();
    items[i].proxy();
    items[i].display();
  }
  server.sendScreen();
}
 
class Item {
 
  float x, y;
  float vx = 0;
  float vy = 0;
  float a = 0;
  float diameter = 0;
  float speed = 0;
  int id;
  Item[] others;
 
  Item(float xin, float yin, float din, int idin, Item[] oin) {
    x = xin;
    y = yin;
    diameter = din;
    int id;
    others = oin;
  }
 
  void display() {
    fill(255, 155);
    noStroke();
    ellipse(x, y, diameter, diameter);
  }
 
  void move() {
    //damping
    x += vx;
    y += vy;
    if (x + diameter/2 > width) {
      x = width - diameter/2;
      vx *= friction;
    }else if (x - diameter/2 < 0) {
      x = diameter/2;
      vx *= friction;
    }
    if (y + diameter/2 > height) {
      y = height - diameter/2;
      vy *= friction;
    }else if (y - diameter/2 < 0) {
      y = diameter/2;
      vy *= friction;
    }
    //follow mousex, mousey
    pushMatrix();
    float px = v1e.x - x;
    float py = v1e.y - y;
    a = atan2(py, px);
    rotate(a);
    speed = random(1, 5);
    x += (cos(a)*speed);
    y += (sin(a)*speed);
    popMatrix();
  }
  //Thanks to Keith Peters
  void proxy() {
    for (int i = id + 1; i < itemNum; i++) {
      //get distance
      float dx = others[i].x - x;
      float dy = others[i].y - y;
      float distance = sqrt(dx*dx + dy*dy);
      float minDist = others[i].diameter/2 + diameter/2;
      //collision test
      if (distance < minDist) {
        float angle = atan2(dy, dx);
        float targetX = x + cos(angle) * minDist;
        float targetY = y + sin(angle) * minDist;
        float ax = (targetX - others[i].x) * 0.05;
        float ay = (targetY - others[i].y) * 0.05;
        vx -= ax;
        vy -= ay;
        others[i].vx += ax;
        others[i].vy += ay;
      }
      //proximity test
      stroke(255, 55);
      strokeWeight(2);
      if (distance < 100) {
        line(x, y, others[i].x, others[i].y);
      }
    }
  }
}
